package com.hh.arome.lib_sscard;

import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.Build;

import com.donsee.bean.SciCardInfo;
import com.donsee.devices.CardReader;
import com.donsee.devices.LogUtils;
import com.hh.arome.basecard.BaseSocialSecurityCardBean;
import com.hh.arome.basecard.BaseSocialSecurityCardReader;
import com.hh.arome.basecard.SocialSecurityCardConfig;

import java.util.HashMap;

/**
 * @Description:
 * @author: Xia WeiHao
 * @date: 2024/6/24
 */
public class DX_SSCardReader extends BaseSocialSecurityCardReader {

    private UsbDevice usbDevice;
    private UsbManager usbManager;
    private PendingIntent mPendingIntent;

    //硬件厂商回复：多合一免驱的，都是这个固定的vid pid
    private int pid = 57407;
    private int vid = 1742;
    private final String ACTION_USB_PERMISSION = "com.donsee.devices.DonseeDeviceActivity.USB_PERMISSION";
    long result = -1;

    private USBReceiver mUsbReceiver;

    private CardReader cardReader;
    private CReaderThread cReaderDataThread;

    private volatile boolean isRuning = false;

    private boolean isStop = false;



    @Override
    public void init(Context context) {
        cardReader = new CardReader(context);
        initUSB(context);
        result = cardReader.Donsee_Open("USB");
        if (result == 0) {
            setmCardReadState(1);
        }
        if (listener != null) {
            listener.onInfoLog(result == 0 ? "社保卡读卡器初始化成功！" : "错误码：" + result + " 社保卡读卡器初始化失败！");
        }
        registerReceiver(context);
    }

    /**
     *
     * desc:初始化USB
     * @author xia weihao
     * @date 2024/6/26
     *
     */
    private void initUSB(Context mContext) {
        usbManager = (UsbManager) mContext.getApplicationContext().getSystemService(Context.USB_SERVICE);
        if (Build.VERSION.SDK_INT >= 31) {
            mPendingIntent = PendingIntent.getBroadcast(mContext, 0, new Intent(ACTION_USB_PERMISSION), PendingIntent.FLAG_IMMUTABLE);
        } else {
            mPendingIntent = PendingIntent.getBroadcast(mContext, 0, new Intent(ACTION_USB_PERMISSION), 0);
        }
        IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
        mContext.registerReceiver(mUsbReceiver, filter);
        HashMap<String, UsbDevice> deviceList = usbManager.getDeviceList();
        if (!deviceList.isEmpty()) {
            for (UsbDevice next : deviceList.values()) {
                if (next.getVendorId() == vid && next.getProductId() == pid) {
                    usbDevice = next;
                    if (!usbManager.hasPermission(usbDevice)) {
                        usbManager.requestPermission(usbDevice, mPendingIntent);
                    } else {
                        result = cardReader.Donsee_Open("USB");
                        if (result == 0) {
                            setmCardReadState(1);
                        }
                    }
                }
            }
        }
        if (usbDevice == null) {
            setmCardReadState(0);
        }
    }


    private class CReaderThread extends Thread {
        private final Object lock = new Object();
        private volatile boolean pause = false;

        public CReaderThread() {
        }

        void pauseThread() {
            pause = true;
        }

        void resumeThread() {
            pause = false;
            synchronized (lock) {
                lock.notifyAll();
            }
        }

        void onPause() {
            synchronized (lock) {
                try {
                    lock.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        @Override
        public void run() {
            super.run();
            while (isRuning) {
                while (pause) onPause();
                readSSCard();
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

            }
        }
    }


    /**
     *
     * desc:启动轮询社保卡
     * @author xia weihao
     * @date 2024/6/26
     *
     */ 
    public void startRead() {
        if (cReaderDataThread == null) {
            cReaderDataThread = new CReaderThread();
        }
        if (!cReaderDataThread.isAlive()) {
            isRuning = true;
            cReaderDataThread.start();
        }

    }

    /**
     *
     * desc:停止轮询读取社保卡
     * @author xia weihao
     * @date 2024/6/26
     *
     */ 
    public void stopRead() {
        if (cReaderDataThread == null) {
//            showString("请先启动读卡");
            return;
        }
        isRuning = false;
        cReaderDataThread.pause = false;
        cReaderDataThread.interrupt();
        cReaderDataThread = null;
    }

    /**
     *
     * desc:读取社保卡数据
     * @author xia weihao
     * @date 2024/6/26
     *
     */ 
    private void readSSCard() {
        SciCardInfo sciCardInfo = new SciCardInfo();
        //nType 1，	有SAM卡返回全部信息   2，	无SAM卡返回卡号
        byte[] errInfor = new byte[256];
        SocialSecurityCardConfig config = getConfig();
        long result = cardReader.Donsee_ReadSSCard(config.getSlotPSAM(), config.getnType(), sciCardInfo, errInfor);

        if (listener != null && isRuning) {
            if (result == 0) {
                BaseSocialSecurityCardBean info = new BaseSocialSecurityCardBean();
                info.cardNumber = sciCardInfo.cardNO;
                info.name = sciCardInfo.name;
                info.sex = sciCardInfo.sex;
                info.nation = sciCardInfo.nation;
                info.idNumber = sciCardInfo.idNO;
                info.birthDate = sciCardInfo.birthDate;
                info.issueDate = sciCardInfo.issueDate;
                info.expireDate = sciCardInfo.expireDate;
                info.cityCode = sciCardInfo.cityCode;
                info.fullVersion = sciCardInfo.fullVersion;
                listener.onSuccess(info);
            } else {
                listener.onFail("读社保卡失败：" + CardReader.getErrorMessage(result));
            }
        }

    }

    /**
     *
     * desc:注册USB监听广播
     * @author xia weihao
     * @date 2024/6/26
     *
     */ 
    public void registerReceiver(Context mContext) {
        IntentFilter filter = new IntentFilter();
        filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
        filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        mUsbReceiver = new USBReceiver();
        mContext.registerReceiver(mUsbReceiver, filter);
    }

    /**
     *
     * desc:USB拔插广播
     * @author xia weihao
     * @date 2024/6/26
     *
     */ 
    private class USBReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            // 这里可以拿到插入的USB设备对象
            LogUtils.d("intent.getAction = " + intent.getAction());
            UsbDevice receiveDevice = intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
            if (receiveDevice != null) {
                /*    private int pid = 57407;  private int vid = 1742;*/
                if (receiveDevice.getProductId() != pid || receiveDevice.getVendorId() != vid)
                    return;
            }
            switch (intent.getAction()) {
                case UsbManager.ACTION_USB_DEVICE_ATTACHED: // 插入USB设备
                    if (result == -1) {
                        cardReader = null;
                        cardReader = new CardReader(context);
                        setmCardReadState(2);
                    }

                    break;
                case UsbManager.ACTION_USB_DEVICE_DETACHED: // 拔出USB设备
                    usbDevice = null;
                    if (result == 0) {
                        setmCardReadState(3);
                        usbDevice = null;
                        cardReader.Donsee_Close();
                        result = -1;
                    }
                    break;
                case ACTION_USB_PERMISSION:
                    UsbDevice usbDevice1 = (UsbDevice) intent.getParcelableExtra("device");
                    if (intent.getBooleanExtra("permission", false) && usbDevice1 != null) {
                        result = cardReader.Donsee_Open("USB");
                        if (result == 0) {
                            setmCardReadState(1);
                        }
                    }
                    break;
                default:
                    break;
            }
        }
    }

    /**
     *
     * desc:销毁资源
     * @author xia weihao
     * @date 2024/6/26
     *
     */ 
    @Override
    public void release(Context context) {
        stopRead();
        if (mUsbReceiver != null) {
            context.unregisterReceiver(mUsbReceiver);
        }
        if (cardReader != null) {
            cardReader.Donsee_Close();
            cardReader.unregisterUSBReceiver();
        }

    }
}
